@@ -1,6 +1,8 @@
+from chalk._validation.validation import Validation
 from chalk._version import __version__
 from chalk.features import Cron, Environments, Tags, description, is_primary, owner, tags
 from chalk.features.resolver import OfflineResolver, OnlineResolver, Resolver, offline, online
+from chalk.features.tag import BranchId, EnvironmentId
 from chalk.logging import chalk_logger
 from chalk.state import State
 from chalk.streams import Windowed, stream, windowed
@@ -12,9 +14,11 @@ realtime = online
 
 __all__ = [
     "AnyDataclass",
+    "BranchId",
     "Cron",
     "CronTab",
     "Duration",
+    "EnvironmentId",
     "Environments",
     "OfflineResolver",
     "OnlineResolver",
@@ -35,4 +39,5 @@ __all__ = [
     "stream",
     "tags",
     "windowed",
+    "Validation",
 ]
