@@ -1,7 +1,7 @@
 import dataclasses
 import zlib
 from copy import deepcopy
-from typing import Any, Dict, List, Literal, Optional, Tuple, TypeVar, Union
+from typing import Any, List, Literal, Optional, Tuple, TypeVar, Union
 
 from chalk._monitoring.charts_enums_codegen import (
     ChartLinkKind,
@@ -11,8 +11,7 @@ from chalk._monitoring.charts_enums_codegen import (
     MetricKind,
     WindowFunctionKind,
 )
-from chalk.features import FeatureWrapper
-from chalk.features.resolver import Resolver, ResolverProtocol
+from chalk.features.resolver import ResolverProtocol
 
 
 @dataclasses.dataclass
@@ -39,7 +38,7 @@ class SeriesBase:
     """
 
     def __init__(
-        self: TSeries,
+        self,
         name: str,
         metric: Union[MetricKind, str],
         window_function: Optional[Union[WindowFunctionKind, str]] = None,
@@ -49,7 +48,7 @@ class SeriesBase:
         self._metric = MetricKind(metric.upper()) if metric else None
         self._filters: List[MetricFilter] = []
         if window_function:
-            if WindowFunctionKind.has_member_key(window_function.upper()):
+            if window_function.upper() in WindowFunctionKind.__members__:
                 window_function_kind = WindowFunctionKind(window_function.upper())
             else:
                 window_function_kind = self._get_window_function_type(window_function.upper())
@@ -98,7 +97,7 @@ class SeriesBase:
         resolver_type: Optional[Union[List[ResolverType], ResolverType]] = None,
         cache_hit: Optional[bool] = None,
         is_null: Optional[bool] = None,
-        equals: Optional[bool] = True,
+        equals: bool = True,
     ) -> TSeries:
         copy = self._copy_with()
         success_dict = {"success": True, "failure": False}
@@ -161,27 +160,27 @@ class SeriesBase:
             copy._filters.append(metric_filter)
         return copy
 
-    def _resolver_name_filter(self: TSeries, *resolvers: Tuple[Union[ResolverProtocol, str]], equals: bool) -> TSeries:
+    def _resolver_name_filter(self: TSeries, *resolvers: Union[ResolverProtocol, str], equals: bool) -> TSeries:
         if not resolvers:
             raise ValueError(f"One or more Chalk Resolvers must be supplied.")
         copy = self._copy_with()
         comparator = ComparatorKind.EQ if equals else ComparatorKind.NEQ
         if len(resolvers) == 1 or not equals:
             for resolver in resolvers:
-                value = resolver.fqn if isinstance(resolver, Resolver) else resolver
+                value = resolver if isinstance(resolver, str) else resolver.fqn
                 metric_filter = MetricFilter(kind=FilterKind.RESOLVER_NAME, comparator=comparator, value=[value])
                 copy._filters.append(metric_filter)
             if len(resolvers) == 1:
                 resolver = resolvers[0]
-                copy._entity_id = resolver.fqn if isinstance(resolver, Resolver) else resolver
+                copy._entity_id = resolver if isinstance(resolver, str) else resolver.fqn
                 copy._entity_kind = ChartLinkKind.resolver
         else:
-            value = [resolver.fqn if isinstance(resolver, Resolver) else resolver for resolver in resolvers]
+            value = [resolver if isinstance(resolver, str) else resolver.fqn for resolver in resolvers]
             metric_filter = MetricFilter(kind=FilterKind.RESOLVER_NAME, comparator=ComparatorKind.ONE_OF, value=value)
             copy._filters.append(metric_filter)
         return copy
 
-    def _string_filter(self: TSeries, *strings: Tuple[str], kind: FilterKind, equals=True) -> TSeries:
+    def _string_filter(self: TSeries, *strings: str, kind: FilterKind, equals: bool = True) -> TSeries:
         if not strings:
             raise ValueError(f"One or more arguments must be supplied for this filter")
         copy = self._copy_with()
@@ -199,7 +198,7 @@ class SeriesBase:
         return copy
 
     def _with_resolver_type_filter(
-        self: TSeries, *resolver_types: Tuple[Literal["online", "offline", "stream"]], equals=True
+        self: TSeries, *resolver_types: Literal["online", "offline", "stream"], equals: bool = True
     ) -> TSeries:
         if not resolver_types:
             raise ValueError(f"One or more resolver types from 'online', 'offline', or 'stream' must be supplied")
@@ -222,8 +221,8 @@ class SeriesBase:
 
     def _true_false_filter(self: TSeries, kind: FilterKind, value: bool) -> TSeries:
         copy = self._copy_with()
-        value = "true" if value else "false"
-        metric_filter = MetricFilter(kind=kind, comparator=ComparatorKind.EQ, value=[value])
+        value_str = "true" if value else "false"
+        metric_filter = MetricFilter(kind=kind, comparator=ComparatorKind.EQ, value=[value_str])
         copy._filters.append(metric_filter)
         return copy
 
@@ -268,10 +267,10 @@ class SeriesBase:
         self_copy = deepcopy(self)
         return self_copy
 
-    def __gt__(self, other) -> ThresholdFunction:
+    def __gt__(self, other: Union[float, int]) -> ThresholdFunction:
         return ThresholdFunction(self, ">", other)
 
-    def __lt__(self, other) -> ThresholdFunction:
+    def __lt__(self, other: Union[float, int]) -> ThresholdFunction:
         return ThresholdFunction(self, "<", other)
 
     def __str__(self) -> str:
