@@ -5,6 +5,7 @@ from datetime import datetime
 from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Protocol, Sequence, Union, overload
 
 import pandas as pd
+import requests
 from typing_extensions import TypeAlias
 
 from chalk.client.exc import (
@@ -208,37 +209,26 @@ class Dataset(Protocol):
     ...     dataset_name='my_dataset'
     ... )
     >>> df = dataset.get_data_as_pandas()
+    >>> df.recompute(features=[User.fraud_score], branch="feature/testing")
     """
 
-    @property
-    def is_finished(self) -> bool:
-        """Whether the most recent `DatasetRevision` is finished or still pending."""
-        ...
+    is_finished: bool
+    """Whether the most recent `DatasetRevision` is finished or still pending."""
 
-    @property
-    def version(self) -> int:
-        """Storage version number of outputs."""
-        ...
+    version: int
+    """Storage version number of outputs."""
 
-    @property
-    def revisions(self) -> Sequence[DatasetRevision]:
-        """A list of all `DatasetRevision` instances belonging to this dataset."""
-        ...
+    revisions: List[DatasetRevision]
+    """A list of all `DatasetRevision` instances belonging to this dataset."""
 
-    @property
-    def dataset_name(self) -> Optional[str]:
-        """The unique name for this dataset, if given."""
-        ...
+    dataset_name: Optional[str]
+    """The unique name for this dataset, if given."""
 
-    @property
-    def dataset_id(self) -> Optional[uuid.UUID]:
-        """The unique UUID for this dataset."""
-        ...
+    dataset_id: Optional[uuid.UUID]
+    """The unique UUID for this dataset."""
 
-    @property
-    def errors(self) -> Optional[List[ChalkError]]:
-        """A list of errors in loading the dataset, if they exist."""
-        ...
+    errors: Optional[List[ChalkError]]
+    """A list of errors in loading the dataset, if they exist."""
 
     def get_data_as_polars(
         self,
@@ -313,7 +303,7 @@ class Dataset(Protocol):
         self,
         path: str,
         num_executors: Optional[int] = None,
-    ):
+    ) -> None:
         """Downloads output files pertaining to the revision to given path.
 
         Datasets are stored in Chalk as sharded Parquet files. With this
@@ -352,6 +342,22 @@ class Dataset(Protocol):
         """
         ...
 
+    def recompute(self, branch: BranchId, features: Optional[List[Union[Feature, Any]]]):
+        """Creates a new revision of this dataset by recomputing the specified features.
+
+        Carries out the new computation on the specified branch.
+
+        Examples
+        --------
+        >>> from chalk.client import ChalkClient
+        >>> dataset = ChalkClient(branch="data_science").offline_query(...)
+        >>> df = dataset.get_data_as_polars()
+        >>> # change some resolvers
+        >>> dataset.recompute(branch="data_science")
+        >>> new_df = dataset.get_data_as_polars() # receive newly computed data
+        """
+        ...
+
 
 class OnlineQueryResult(Protocol):
     data: List[FeatureResult]
@@ -419,7 +425,7 @@ class ChalkClient:
     """
 
     @overload
-    def __init__(self):
+    def __init__(self, session: Optional[requests.Session] = None):
         """Create a `ChalkClient` with environment variables or credentials in `~/.chalk.yml`.
 
         The client will first check for the presence of the environment variables
@@ -434,6 +440,13 @@ class ChalkClient:
         optionally override the API server and environment by setting the
         environment variables `CHALK_API_SERVER` and `CHALK_ENVIRONMENT`.
 
+        Parameters
+        ----------
+        session
+            A `requests.Session` to use for all requests. If not provided,
+            a new session will be created.
+
+
         Raises
         ------
         ChalkAuthException
@@ -451,6 +464,8 @@ class ChalkClient:
         client_secret: str,
         environment: Optional[EnvironmentId] = None,
         api_server: Optional[str] = None,
+        branch: Optional[BranchId] = None,
+        session: Optional[requests.Session] = None,
     ):
         """Create a `ChalkClient` with the given credentials.
 
@@ -473,6 +488,14 @@ class ChalkClient:
             using a Chalk Dedicated deployment. If not present, the client
             will check for the presence of the environment variable
             `CHALK_API_SERVER`, and use that if found.
+        branch
+            If specified, Chalk will route all requests from this client
+            instance to the relevant branch. Some methods allow you to
+            override this instance-level branch configuration by passing
+            in a `branch` argument.
+        session
+            A `requests.Session` to use for all requests. If not provided,
+            a new session will be created.
 
         Raises
         ------
@@ -490,12 +513,14 @@ class ChalkClient:
         client_secret: Optional[str] = None,
         environment: Optional[EnvironmentId] = None,
         api_server: Optional[str] = None,
+        branch: Optional[BranchId] = None,
+        session: Optional[requests.Session] = None,
     ):
         """Create a `ChalkClient`. Clients can either be created
         explicitly, with environment variables, or using credentials
         in `~/.chalk.yml`.
 
-        The `__init__` method specifies two overloads for this purpose.
+        The `__init__` method specifies overloads for this purpose.
         See the overloaded methods for details.
         """
         ...
