@@ -5,7 +5,22 @@ import time
 import uuid
 import warnings
 from datetime import datetime, timezone
-from typing import TYPE_CHECKING, Any, List, Literal, Mapping, Optional, Sequence, Type, TypeVar, Union, cast, overload
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    List,
+    Literal,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+    overload,
+)
 from urllib.parse import urljoin
 
 import pandas as pd
@@ -13,6 +28,7 @@ import requests
 from pydantic import BaseModel, ValidationError
 from requests import HTTPError
 from requests import JSONDecodeError as RequestsJSONDecodeError
+from requests.sessions import SessionRedirectMixin
 
 from chalk._version import __version__ as chalkpy_version
 from chalk.client import ChalkClient, OnlineQueryResult
@@ -24,7 +40,9 @@ from chalk.client.exc import (
     ChalkDatasetDownloadException,
     ChalkOfflineQueryException,
     ChalkOnlineQueryException,
+    ChalkRecomputeDatasetException,
     ChalkResolverRunException,
+    ChalkUpdateResolverException,
     ChalkWhoAmIException,
 )
 from chalk.client.models import (
@@ -33,7 +51,9 @@ from chalk.client.models import (
     ComputeResolverOutputResponse,
     CreateOfflineQueryJobRequest,
     CreateOfflineQueryJobResponse,
+    DatasetRecomputeRequest,
     DatasetResponse,
+    DatasetRevisionResponse,
     ErrorCode,
     ExchangeCredentialsRequest,
     ExchangeCredentialsResponse,
@@ -46,10 +66,12 @@ from chalk.client.models import (
     OnlineQueryResponse,
     ResolverRunResponse,
     TriggerResolverRunRequest,
+    UpdateResolverResponse,
     WhoAmIResponse,
 )
 from chalk.config.auth_config import load_token
 from chalk.features import DataFrame, Feature, FeatureNotFoundException, ensure_feature
+from chalk.features._encoding.inputs import recursive_encode
 from chalk.features.pseudofeatures import CHALK_TS_FEATURE
 from chalk.features.tag import BranchId, EnvironmentId
 from chalk.utils.log_with_context import get_logger
@@ -69,6 +91,7 @@ class _ChalkClientConfig(BaseModel):
     client_secret: str
     api_server: str
     active_environment: Optional[str]
+    branch: Optional[BranchId]
 
 
 def _to_offline_query_input(
@@ -96,7 +119,7 @@ def _to_offline_query_input(
     if input_times is None:
         input_times = datetime.now(timezone.utc)
     if isinstance(input_times, datetime):
-        input_times = [input_times for _ in range(len(input))]
+        input_times = [input_times for _ in range(len(pd_dataframe))]
     local_tz = datetime.now(timezone.utc).astimezone().tzinfo
 
     input_times = [x.replace(tzinfo=local_tz) if x.tzinfo is None else x for x in input_times]
@@ -180,6 +203,8 @@ class ChalkAPIClientImpl(ChalkClient):
     __name__ = "ChalkClient"
     __qualname__ = "chalk.client.ChalkClient"
 
+    _latest_client: Optional[ChalkAPIClientImpl]
+
     def __repr__(self):
         return f"chalk.client.ChalkClient<{self._config.name}>"
 
@@ -194,9 +219,11 @@ class ChalkAPIClientImpl(ChalkClient):
         client_secret: Optional[str] = None,
         environment: Optional[EnvironmentId] = None,
         api_server: Optional[str] = None,
+        branch: Optional[BranchId] = None,
         _skip_cache: bool = False,
+        session: Optional[requests.Session] = None,
     ):
-        self.session = requests.Session()
+        self.session: requests.Session = session or requests.Session()
         token = load_token(
             client_id=client_id,
             client_secret=client_secret,
@@ -212,6 +239,7 @@ class ChalkAPIClientImpl(ChalkClient):
             client_id=token.clientId,
             client_secret=token.clientSecret,
             api_server=token.apiServer or "https://api.chalk.ai",
+            branch=branch,
             active_environment=token.activeEnvironment,
         )
 
@@ -225,6 +253,8 @@ class ChalkAPIClientImpl(ChalkClient):
         self._exchanged_credentials = False
         self._primary_environment = None
 
+        self.__class__._latest_client = self
+
     def _exchange_credentials(self):
         _logger.debug("Performing a credentials exchange")
         resp = self.session.post(
@@ -264,6 +294,9 @@ class ChalkAPIClientImpl(ChalkClient):
             headers["X-Chalk-Preview-Deployment"] = preview_deployment_id
         if branch is not None:
             headers["X-Chalk-Branch-Id"] = branch
+        elif self._config.branch is not None:
+            headers["X-Chalk-Branch-Id"] = self._config.branch
+
         return headers
 
     @staticmethod
@@ -306,6 +339,7 @@ class ChalkAPIClientImpl(ChalkClient):
         exception_cls: Type[ChalkBaseException],
         preview_deployment_id: Optional[str],
         branch: Optional[BranchId],
+        data: Optional[bytes] = None,
         **exception_kwargs: Any,
     ) -> T:
         # Track whether we already exchanged credentials for this request
@@ -323,12 +357,12 @@ class ChalkAPIClientImpl(ChalkClient):
         )
         url = urljoin(self._config.api_server, uri)
         json_body = json and json.dict()
-        r = self.session.request(method=method, headers=headers, url=url, json=json_body)
+        r = self.session.request(method=method, headers=headers, url=url, json=json_body, data=data)
         if r.status_code in (401, 403) and not exchanged_credentials:
             # It is possible that credentials expired, or that we changed permissions since we last
             # got a token. Exchange them and try again
             self._exchange_credentials()
-            r = self.session.request(method=method, headers=headers, url=url, json=json_body)
+            r = self.session.request(method=method, headers=headers, url=url, json=json_body, data=data)
 
         try:
             r.raise_for_status()
@@ -386,19 +420,7 @@ class ChalkAPIClientImpl(ChalkClient):
     ) -> OnlineQueryResponseImpl:
         environment = environment or (context and context.environment)
         tags = tags or (context and context.tags)
-        warnings: List[str] = []
-        encoded_inputs = {}
-        for feature, value in input.items():
-            try:
-                feature = ensure_feature(feature)
-            except FeatureNotFoundException:
-                warnings.append(f"Input {feature} not recognized. JSON encoding {feature} and requesting anyways")
-                encoded_inputs[feature] = value
-            else:
-                encoded_inputs[feature.fqn] = feature.converter.from_rich_to_json(
-                    value,
-                    missing_value_strategy="error",
-                )
+        encoded_inputs, encoding_warnings = recursive_encode(input)
 
         outputs: List[str] = [str(feature) for feature in output]
 
@@ -426,7 +448,7 @@ class ChalkAPIClientImpl(ChalkClient):
             preview_deployment_id=preview_deployment_id,
             branch=branch,
         )
-        return OnlineQueryResponseImpl(data=resp.data, errors=resp.errors or [], warnings=warnings)
+        return OnlineQueryResponseImpl(data=resp.data, errors=resp.errors or [], warnings=encoding_warnings)
 
     def offline_query(
         self,
@@ -679,6 +701,7 @@ class ChalkAPIClientImpl(ChalkClient):
         output_id: bool,
         output_ts: bool,
         urls_only: Literal[True],
+        branch: Optional[BranchId],
     ) -> List[str]:
         ...
 
@@ -691,12 +714,10 @@ class ChalkAPIClientImpl(ChalkClient):
         output_id: bool,
         output_ts: bool,
         urls_only: bool = False,
+        branch: Optional[BranchId] = None,
     ) -> Union[pl.DataFrame, pl.LazyFrame, List[str]]:
         while True:
-            status = self._get_job_status(
-                job_id=job_id,
-                environment=context and context.environment,
-            )
+            status = self._get_job_status(job_id=job_id, environment=context and context.environment, branch=branch)
             if status.is_finished:
                 break
             time.sleep(0.5)
@@ -712,6 +733,34 @@ class ChalkAPIClientImpl(ChalkClient):
             lazy=lazy,
         )
 
+    def _recompute_dataset(
+        self,
+        dataset_name: str | None,
+        dataset_id: uuid.UUID | None,
+        revision_id: uuid.UUID | None,
+        features: List[Union[str, Any]],
+        branch: BranchId,
+        environment: Optional[EnvironmentId],
+    ) -> DatasetRevisionResponse:
+        request = DatasetRecomputeRequest(
+            dataset_name=dataset_name,
+            dataset_id=str(dataset_id) if dataset_id is not None else None,
+            revision_id=str(revision_id) if revision_id is not None else None,
+            features=features,
+            branch=branch,
+        )
+        return self._request(
+            method="POST",
+            uri="/v1/dataset/recompute",
+            json=request,
+            response=DatasetRevisionResponse,
+            environment_override=environment,
+            exception_cls=ChalkRecomputeDatasetException,
+            preview_deployment_id=None,
+            branch=branch,
+            dataset_name=str(dataset_id),
+        )
+
     def _create_dataset_job(
         self,
         optional_output: List[str],
@@ -847,9 +896,7 @@ class ChalkAPIClientImpl(ChalkClient):
         return response
 
     def _get_job_status(
-        self,
-        job_id: uuid.UUID,
-        environment: Optional[EnvironmentId],
+        self, job_id: uuid.UUID, environment: Optional[EnvironmentId], branch: Optional[BranchId]
     ) -> GetOfflineQueryJobResponse:
         return self._request(
             method="GET",
@@ -859,7 +906,7 @@ class ChalkAPIClientImpl(ChalkClient):
             json=None,
             exception_cls=ChalkOfflineQueryException,
             preview_deployment_id=None,
-            branch=None,
+            branch=branch,
         )
 
     def _get_dataset(self, dataset_name: str, environment: Optional[EnvironmentId]) -> DatasetResponse:
@@ -874,3 +921,18 @@ class ChalkAPIClientImpl(ChalkClient):
             preview_deployment_id=None,
             branch=None,
         )
+
+    def _send_updated_resolver(
+        self, environment: Optional[EnvironmentId], pickled_resolver: bytes
+    ) -> UpdateResolverResponse:
+        return self._request(
+            method="POST",
+            uri=f"/v1/update_resolver",
+            response=UpdateResolverResponse,
+            json=None,
+            data=pickled_resolver,
+            environment_override=environment,
+            exception_cls=ChalkUpdateResolverException,
+            preview_deployment_id=None,
+            branch=None,
+        )
