@@ -83,6 +83,24 @@ class ChalkResolverRunException(ChalkBaseException):
         return "Resolver run failed"
 
 
+class ChalkUpdateResolverException(ChalkBaseException):
+    """Raised when failing to update a resolver on the fly from a notebook"""
+
+    @property
+    def message(self) -> str:
+        return "Failed to update resolver."
+
+
+class ChalkRecomputeDatasetException(ChalkBaseException):
+    def __init__(self, dataset_name: str, errors: Optional[List[ChalkError]] = None):
+        self.dataset_name = dataset_name
+        super().__init__(errors)
+
+    @property
+    def message(self) -> str:
+        return f"Failed to recompute dataset '{self.dataset_name}'"
+
+
 class ChalkDatasetDownloadException(ChalkBaseException):
     def __init__(self, dataset_name: str, errors: Optional[List[ChalkError]] = None):
         self.dataset_name = dataset_name
