@@ -150,19 +150,21 @@ class DataFrameMeta(type):
         from chalk.features.feature_field import Feature
 
         cls = cast(Type[DataFrame], cls)
-        columns: Set[Feature] = set()
+        columns: List[Feature] = []
         for x in cls.__columns__:
             assert isinstance(x, Feature)
             # If a feature is directly specified, allow has-ones. But still do not allow has-many features
             assert not x.is_has_many, "Has-many features are not allowed to be specified within a DataFrame"
-            columns.add(x)
+            if x not in columns:
+                columns.append(x)
         if cls.__references_feature_set__ is not None:
             # Only include the first-level feature types
             # Do not recurse has-ones and has-many as that could create an infinite loop
             for x in cls.__references_feature_set__.features:
                 assert isinstance(x, Feature)
                 if not x.is_has_many and not x.is_has_one:
-                    columns.add(x)
+                    if x not in columns:
+                        columns.append(x)
         return tuple(columns)
 
     @property
@@ -626,7 +628,7 @@ class DataFrameImpl(metaclass=DataFrameMeta):
             pyarrow_options=pyarrow_options,
         )
         if new_columns is not None:
-            data = data.rename({c: new_columns for c in data.columns})
+            data = data.rename({c: new_columns[i] for i, c in enumerate(data.columns)})
         return cls(data)
 
     @classmethod
@@ -639,6 +641,7 @@ class DataFrameImpl(metaclass=DataFrameMeta):
                 Dict[int, Union[str, Feature, Any]],
             ]
         ] = None,
+        use_pyarrow: bool = False,
     ) -> TDataFrame:
         try:
             import polars as pl
@@ -654,6 +657,7 @@ class DataFrameImpl(metaclass=DataFrameMeta):
             source=path,
             columns=cols_to_select,
             storage_options=DataFrame._get_storage_options(),
+            use_pyarrow=use_pyarrow,
         )
         if new_columns is not None:
             data = data.rename({c: new_columns[i] for i, c in enumerate(data.columns)})
@@ -735,6 +739,56 @@ class DataFrameImpl(metaclass=DataFrameMeta):
         )
         return cls(data)
 
+    @classmethod
+    def read_avro(
+        cls: Type[TDataFrame],
+        path: Union[str, pathlib.Path],
+    ) -> TDataFrame:
+        """Read a .avro file as a `DataFrame`.
+
+        Parameters
+        ----------
+        path
+            The path to the `.avro` file. This may be a S3 or GCS
+            storage url.
+
+        Returns
+        -------
+        TDataFrame
+            A `DataFrame` with the contents of the file loaded as features.
+
+        Examples
+        --------
+        >>> values = DataFrame.read_avro(
+        ...     "s3://...",
+        ... )
+        """
+        try:
+            import polars as pl
+        except ImportError:
+            raise missing_dependency_exception("chalkpy[runtime]")
+
+        data = None
+        if (
+            (type(path) == "string" and path.startswith("s3"))
+            or (type(path) == "PosixPath")
+            and (str(path).startswith("s3"))
+        ):
+            try:
+                import fsspec
+            except ImportError:
+                raise missing_dependency_exception("fsspec")
+            try:
+                import io
+            except ImportError:
+                raise missing_dependency_exception("io")
+            with fsspec.open(path, "rb") as f:
+                buffer = io.BytesIO(initial_bytes=f.read())
+                data = pl.read_avro(buffer)
+        else:
+            data = pl.read_avro(file=path)
+        return cls(data)
+
     #############
     # Aggregation
     #############
