@@ -19,8 +19,8 @@ Features named `id` on feature classes without an explicit primary
 feature are declared primary keys by default, and don't need to be
 marked with `Primary`.
 
-If you have primary key feature with a name other than `id`, you
-can use this marker to indicate the primary key.
+If you have primary key feature with a name other than `id`,
+you can use this marker to indicate the primary key.
 
 Examples
 --------
