@@ -41,7 +41,7 @@ from chalk.utils.log_with_context import get_logger, get_logging_context
 from chalk.utils.missing_dependency import missing_dependency_exception
 
 if TYPE_CHECKING:
-    from sqlalchemy.engine import URL, Engine
+    from sqlalchemy.engine import URL, Connection, Engine
     from sqlalchemy.orm import Session
     from sqlalchemy.sql import Select
     from sqlalchemy.sql.elements import Label
@@ -130,7 +130,7 @@ class TableIngestMixIn(TableIngestProtocol):
 class BaseSQLSource(BaseSQLSourceProtocol):
     registry: ClassVar[List["BaseSQLSource"]] = []
 
-    def __init__(self, name: Optional[str]):
+    def __init__(self, name: Optional[str], engine_args: Optional[Dict[str, Any]]):
         try:
             import sqlalchemy
         except ImportError:
@@ -141,6 +141,15 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         self._resolver_and_sqlfile_to_sqlstring: Dict[Tuple[str, str], str] = {}
         self.registry.append(self)
         self.name = name
+        if engine_args is None:
+            engine_args = {}
+        if self.name is not None:
+            for k, v in self._load_env_engine_args(name=self.name).items():
+                engine_args.setdefault(k, v)
+        engine_args.setdefault("pool_pre_ping", env_var_bool("USE_CLIENT_POOL_PRE_PING"))
+        engine_args.setdefault("url", self.local_engine_url())
+        engine_args.setdefault("echo", CHALK_QUERY_LOGGING)
+        self.engine_args = engine_args
 
     def query_sql_file(
         self,
@@ -224,10 +233,8 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         self,
         finalized_query: FinalizedChalkQuery,
         columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
-        connection: Optional[Any] = None,
+        connection: Optional[Connection] = None,
     ) -> pa.Table:
-        from sqlalchemy.engine import Connection
-
         try:
             return self.execute_query_efficient(finalized_query, columns_to_converters, connection)
         except NotImplementedError:
@@ -242,8 +249,15 @@ class BaseSQLSource(BaseSQLSourceProtocol):
             log_level = e.log_level
             _logger.log(log_level, str(e))
 
+        return self.execute_query_inefficient(finalized_query, columns_to_converters, connection)
+
+    def execute_query_inefficient(
+        self,
+        finalized_query: FinalizedChalkQuery,
+        columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
+        connection: Optional[Connection],
+    ):
         with self.get_engine().connect() if connection is None else contextlib.nullcontext(connection) as cnx:
-            assert isinstance(cnx, Connection)
             res = cnx.execute(finalized_query.query, finalized_query.params)
             desc = res.cursor.description
             result_columns: list[str] = [col[0] for col in desc]
@@ -267,7 +281,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         self,
         finalized_query: FinalizedChalkQuery,
         columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
-        connection: Optional[Any] = None,
+        connection: Optional[Connection],
     ) -> pa.Table:
         raise NotImplementedError()
 
@@ -283,7 +297,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         query_string = compiled_query.string
         return query_string, compiled_query.positiontup or [], compiled_query.params
 
-    def load_env_engine_args(self, name: str) -> Mapping[str, Any]:
+    def _load_env_engine_args(self, name: str) -> Mapping[str, Any]:
         """
         Loads additional engine arguments from env var "{name}_ENGINE_ARGUMENTS"
         """
@@ -297,6 +311,35 @@ class BaseSQLSource(BaseSQLSourceProtocol):
             return extra_args
 
     def get_engine(self, additional_engine_args: Optional[Dict[str, Any]] = None) -> Engine:
+        if additional_engine_args is not None:
+            # Deprecating the additional engine args, and instead requiring they are specified when creating the engine,
+            # since otherwise the engine won't be cached
+            # If we need different types of engines for a given SQL source, we should probably add explicit methods
+            # e.g. get_read_engine, get_write_engine, etc...
+            warnings.warn(
+                DeprecationWarning(
+                    (
+                        "The argument `additional_engine_args` is deprecated for the method `SQLSource.get_engine`. "
+                        "Instead, specify the arguments when constructing the SQL source -- "
+                        "e.g. `PostgresSQLSource(engine_args=...)`"
+                    )
+                )
+            )
+        isolation_level = self.engine_args.get("isolation_level")
+        if isolation_level == "AUTOCOMMIT":
+            warnings.warn(
+                UserWarning(
+                    (
+                        f"The SQL engine '{self.name}' is being created with the AUTOCOMMIT transaction isolation level, which helps improve "
+                        "performance for SELECT statements by avoiding unnecessary transactions. If a different transaction level is needed for an "
+                        "individual connection, use the `execution_options` method when retrieving a connection -- e.g. "
+                        "`with get_engine().connect().execution_options(isolation_level='REPEATABLE READ') as cnx: ...`. "
+                        "For more information, please see "
+                        "https://docs.sqlalchemy.org/en/20/core/connections.html#setting-isolation-level-or-dbapi-autocommit-for-a-connection"
+                    )
+                )
+            )
+
         if additional_engine_args is None:
             return self._get_engine_default()
         else:
@@ -305,24 +348,16 @@ class BaseSQLSource(BaseSQLSourceProtocol):
     def _get_engine_with_args(self, additional_engine_args: Dict[str, Any]) -> Engine:
         from sqlalchemy.engine import create_engine
 
-        args = self.engine_args()
+        args = dict(self.engine_args)
         args.update(additional_engine_args)
 
-        return create_engine(
-            url=self.local_engine_url(),
-            echo=CHALK_QUERY_LOGGING,
-            **args,
-        )
+        return create_engine(**args)
 
     @functools.lru_cache(None)
     def _get_engine_default(self) -> Engine:
         from sqlalchemy.engine import create_engine
 
-        return create_engine(
-            url=self.local_engine_url(),
-            echo=CHALK_QUERY_LOGGING,
-            **self.engine_args(),
-        )
+        return create_engine(**self.engine_args)
 
     def raw_session(self) -> Session:
         from sqlalchemy.orm import Session
