@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import re
 from datetime import timedelta
 from re import Pattern
@@ -63,7 +65,9 @@ One of:
 """
 
 
-def timedelta_to_duration(t: timedelta) -> Duration:
+def timedelta_to_duration(t: timedelta) -> str:
+    if t == timedelta.max:
+        return "infinity"
     s = ""
     if t.days > 0:
         s += f"{t.days}d"
@@ -82,6 +86,9 @@ _kwarg_to_regex: Mapping[str, Pattern] = {
 
 
 def parse_chalk_duration(s: str) -> timedelta:
+    if s == "infinity":
+        return timedelta.max
+
     def parse(regex: Pattern) -> Tuple[int, str]:
         matched = regex.search(s)
         if matched is None:
